@@ -45,7 +45,7 @@
 #include <sys/capability.h>
 #endif
 
-RCSID("$Id: login.c,v 1.120.2.1 1999/08/19 13:38:31 assar Exp $");
+RCSID("$Id: login.c,v 1.120.2.2 1999/09/02 08:55:26 joda Exp $");
 
 #ifdef OTP
 #include <otp.h>
@@ -754,8 +754,17 @@ main(int argc, char **argv)
 	/* perhaps work some magic */
 	if(do_osfc2_magic(pwd->pw_uid))
 	    sleepexit(1);
-#ifdef HAVE_SGI_GETCAPABILITYBYNAME
-	/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */
+#if defined(HAVE_SGI_GETCAPABILITYBYNAME) && defined(HAVE_CAP_SET_PROC)
+	/* XXX SGI capability hack IRIX 6.x (x >= 0?) has something
+	   called capabilities, that allow you to give away
+	   permissions (such as chown) to specific processes. From 6.5
+	   this is default on, and the default capability set seems to
+	   not always be the empty set. The problem is that the
+	   runtime linker refuses to do just about anything if the
+	   process has *any* capabilities set, so we have to remove
+	   them here (unless otherwise instructed by /etc/capability).
+	   In IRIX < 6.5, these functions was called sgi_cap_setproc,
+	   etc, but we ignore this fact (it works anyway). */
 	{
 	    struct user_cap *ucap = sgi_getcapabilitybyname(pwd->pw_name);
 	    cap_t cap;
