@@ -42,7 +42,7 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Id: rshd.c,v 1.53 1998/02/10 21:12:10 bg Exp $");
+RCSID("$Id: rshd.c,v 1.58 1999/06/17 18:49:43 assar Exp $");
 
 extern char *__rcmd_errstr; /* syslog hook from libc/net/rcmd.c. */
 extern int __check_rhosts_file;
@@ -197,7 +197,7 @@ doit(struct sockaddr_in *fromp)
     int one = 1;
     const char *errorhost = "";
     char *errorstr;
-    char *cp, sig, buf[BUFSIZ];
+    char *cp, sig, buf[DES_RW_MAXWRITE];
     char cmdbuf[NCARGS+1], locuser[16], remuser[16];
     char remotehost[2 * MaxHostNameLen + 1];
 
@@ -369,9 +369,9 @@ doit(struct sockaddr_in *fromp)
     } else
 
 	if (errorstr ||
-	    pwd->pw_passwd != 0 && *pwd->pw_passwd != '\0' &&
+	    (pwd->pw_passwd != 0 && *pwd->pw_passwd != '\0' &&
 	    iruserok(fromp->sin_addr.s_addr, pwd->pw_uid == 0,
-		     remuser, locuser) < 0) {
+		     remuser, locuser) < 0)) {
 	    if (__rcmd_errstr)
 		syslog(LOG_INFO|LOG_AUTH,
 		       "%s@%s as %s: permission denied (%s). cmd='%.80s'",
@@ -560,19 +560,18 @@ doit(struct sockaddr_in *fromp)
     if (setpcred (pwd->pw_name, NULL) == -1)
 	syslog(LOG_ERR, "setpcred() failure: %m");
 #endif /* HAVE_SETPCRED */
+    if(do_osfc2_magic(pwd->pw_uid))
+	exit(1);
     setgid((gid_t)pwd->pw_gid);
     initgroups(pwd->pw_name, pwd->pw_gid);
     setuid((uid_t)pwd->pw_uid);
-    strncat(homedir, pwd->pw_dir, sizeof(homedir)-6);
+    strcat_truncate(homedir, pwd->pw_dir, sizeof(homedir));
 
-    /* Need to extend path to find rcp */
-    strncat(path, BINDIR, sizeof(path)-1);
-    strncat(path, ":", sizeof(path)-1);
-    strncat(path, _PATH_DEFPATH, sizeof(path)-1);
-    path[sizeof(path)-1] = '\0';
+    /* Need to prepend path with BINDIR (/usr/athena/bin) to find rcp */
+    snprintf(path, sizeof(path), "PATH=%s:%s", BINDIR, _PATH_DEFPATH);
 
-    strncat(shell, pwd->pw_shell, sizeof(shell)-7);
-    strncat(username, pwd->pw_name, sizeof(username)-6);
+    strcat_truncate(shell, pwd->pw_shell, sizeof(shell));
+    strcat_truncate(username, pwd->pw_name, sizeof(username));
     cp = strrchr(pwd->pw_shell, '/');
     if (cp)
 	cp++;
