@@ -10,7 +10,7 @@
  */
 
 #include "sample.h"
-RCSID("$Id: simple_client.c,v 1.12 1997/12/05 01:26:56 assar Exp $");
+RCSID("$Id: simple_client.c,v 1.13 1998/06/09 19:24:39 joda Exp $");
 
 #define MSG "hi, Jennifer!"		/* message text */
 
@@ -174,12 +174,12 @@ main(int argc, char **argv)
 
   set_progname (argv[0]);
 
-  strcpy (service, SAMPLE_SERVICE);
+  strcpy_truncate (service, SAMPLE_SERVICE, sizeof(service));
 
   while ((c = getopt(argc, argv, "s:p:")) != EOF)
     switch(c) {
     case 's' :
-      strncpy (service, optarg, sizeof(service));
+      strcpy_truncate (service, optarg, sizeof(service));
       break;
     case 'p' :
       serv = getservbyname (optarg, "tcp");
