@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: create_death_packet.c,v 1.7 1996/10/27 16:29:33 joda Exp $");
+RCSID("$Id: create_death_packet.c,v 1.9 1998/06/09 19:25:17 joda Exp $");
 
 /*
  * This routine creates a packet to type AUTH_MSG_DIE which is sent to
@@ -74,11 +74,29 @@ krb_create_death_packet(char *a_name)
     KTEXT pkt = &pkt_st;
 
     unsigned char *p = pkt->dat;
-    
-    p += krb_put_int(KRB_PROT_VERSION, p, 1);
-    p += krb_put_int(AUTH_MSG_DIE, p, 1);
-    
-    p += krb_put_string(a_name, p);
+    int tmp;
+    int rem = sizeof(pkt->dat);
+
+    pkt->length = 0;
+
+    tmp = krb_put_int(KRB_PROT_VERSION, p, rem, 1);
+    if (tmp < 0)
+	return NULL;
+    p += tmp;
+    rem -= tmp;
+
+    tmp = krb_put_int(AUTH_MSG_DIE, p, rem, 1);
+    if (tmp < 0)
+	return NULL;
+    p += tmp;
+    rem -= tmp;
+
+    tmp = krb_put_string(a_name, p, rem);
+    if (tmp < 0)
+	return NULL;
+    p += tmp;
+    rem -= tmp;
+
     pkt->length = p - pkt->dat;
     return pkt;
 }
