@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -38,9 +38,9 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: getaddrs.c,v 1.24 1998/04/26 15:10:44 joda Exp $");
+RCSID("$Id: getaddrs.c,v 1.26.2.1 1999/07/22 03:15:33 assar Exp $");
 
-#if defined(HAVE_SYS_IOCTL_H) && SunOS != 4
+#if defined(HAVE_SYS_IOCTL_H) && SunOS != 40
 #include <sys/ioctl.h>
 #endif
 #ifdef HAVE_NET_IF_H
@@ -81,34 +81,47 @@ k_get_all_addrs (struct in_addr **l)
      return 1;
 #else
      int fd;
-     char buf[BUFSIZ];
+     char *inbuf = NULL;
+     size_t in_len = 8192;
      struct ifreq ifreq;
      struct ifconf ifconf;
      int num, j;
      char *p;
      size_t sz;
 
+     *l = NULL;
      fd = socket(AF_INET, SOCK_DGRAM, 0);
      if (fd < 0)
 	  return -1;
 
-     ifconf.ifc_len = sizeof(buf);
-     ifconf.ifc_buf = buf;
-     if(ioctl(fd, SIOCGIFCONF, &ifconf) < 0)
-	  return -1;
+     for(;;) {
+	 void *tmp;
+
+	 tmp = realloc (inbuf, in_len);
+	 if (tmp == NULL)
+	     goto fail;
+	 inbuf = tmp;
+
+	 ifconf.ifc_len = in_len;
+	 ifconf.ifc_buf = inbuf;
+
+	 if(ioctl(fd, SIOCGIFCONF, &ifconf) < 0)
+	     goto fail;
+	 if(ifconf.ifc_len + sizeof(ifreq) < in_len)
+	     break;
+	 in_len *= 2;
+     }
      num = ifconf.ifc_len / sizeof(struct ifreq);
      *l = malloc(num * sizeof(struct in_addr));
-     if(*l == NULL) {
-	  close (fd);
-	  return -1;
-     }
+     if(*l == NULL)
+	 goto fail;
 
      j = 0;
      ifreq.ifr_name[0] = '\0';
      for (p = ifconf.ifc_buf; p < ifconf.ifc_buf + ifconf.ifc_len; p += sz) {
           struct ifreq *ifr = (struct ifreq *)p;
 	  sz = sizeof(*ifr);
-#ifdef SOCKADDR_HAS_SA_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
 	  sz = max(sz, sizeof(ifr->ifr_name) + ifr->ifr_addr.sa_len);
 #endif
 
@@ -123,9 +136,20 @@ k_get_all_addrs (struct in_addr **l)
 	      ifreq = *ifr;
 	  }
      }
-     if (j != num)
-	 *l = realloc (*l, j * sizeof(struct in_addr));
+     if (j != num) {
+	 void *tmp;
+	 tmp = realloc (*l, j * sizeof(struct in_addr));
+	 if(tmp == NULL)
+	     goto fail;
+	 *l = tmp;
+     }
      close (fd);
+     free(inbuf);
      return j;
+fail:
+     close(fd);
+     free(inbuf);
+     free(*l);
+     return -1;
 #endif /* SIOCGIFCONF */
 }
