@@ -22,17 +22,15 @@ or implied warranty.
 #include "krb_locl.h"
 #include <base64.h>
 
-RCSID("$Id: send_to_kdc.c,v 1.56 1998/05/31 16:46:16 joda Exp $");
+RCSID("$Id: send_to_kdc.c,v 1.69 1999/06/29 21:18:09 bg Exp $");
 
 struct host {
     struct sockaddr_in addr;
     enum krb_host_proto proto;
 };
 
-static const char *prog = "send_to_kdc";
-static send_recv(KTEXT pkt, KTEXT rpkt, int f,
-		 struct sockaddr_in *adr, struct host *addrs,
-		 int h_hosts);
+static int send_recv(KTEXT pkt, KTEXT rpkt, int f,
+		     struct sockaddr_in *adr);
 
 /*
  * send_to_kdc() sends a message to the Kerberos authentication
@@ -64,6 +62,8 @@ static send_recv(KTEXT pkt, KTEXT rpkt, int f,
 /* always use the admin server */
 static int krb_use_admin_server_flag = 0;
 
+static int client_timeout = -1;
+
 int
 krb_use_admin_server(int flag)
 {
@@ -73,7 +73,7 @@ krb_use_admin_server(int flag)
 }
 
 int
-send_to_kdc(KTEXT pkt, KTEXT rpkt, char *realm)
+send_to_kdc(KTEXT pkt, KTEXT rpkt, const char *realm)
 {
     int i;
     int no_host; /* was a kerberos host found? */
@@ -85,6 +85,21 @@ send_to_kdc(KTEXT pkt, KTEXT rpkt, char *realm)
     struct krb_host *k_host;
     struct host *hosts = malloc(sizeof(*hosts));
 
+    if (client_timeout == -1) {
+	const char *to;
+
+	client_timeout = CLIENT_KRB_TIMEOUT;
+	to = krb_get_config_string ("kdc_timeout");
+	if (to != NULL) {
+	    int tmp;
+	    char *end;
+
+	    tmp = strtol (to, &end, 0);
+	    if (end != to)
+		client_timeout = tmp;
+	}
+    }
+
     if (hosts == NULL)
 	return SKDC_CANT;
 
@@ -93,11 +108,11 @@ send_to_kdc(KTEXT pkt, KTEXT rpkt, char *realm)
      * local realm.
      */
     if (realm)
-	strcpy(lrealm, realm);
+	strcpy_truncate(lrealm, realm, REALM_SZ);
     else
 	if (krb_get_lrealm(lrealm,1)) {
 	    if (krb_debug)
-		krb_warning("%s: can't get local realm\n", prog);
+		krb_warning("send_to_kdc: can't get local realm\n");
 	    return(SKDC_CANT);
 	}
     if (krb_debug)
@@ -106,9 +121,14 @@ send_to_kdc(KTEXT pkt, KTEXT rpkt, char *realm)
     no_host = 1;
     /* get an initial allocation */
     n_hosts = 0;
-    for (i = 1; (k_host = krb_get_host(i, lrealm, krb_use_admin_server_flag)); 
+    for (i = 1;
+	 (k_host = krb_get_host(i, lrealm, krb_use_admin_server_flag)); 
 	 ++i) {
 	char *p;
+	char **addr_list;
+	int j;
+	int n_addrs;
+	struct host *tmp;
 
         if (krb_debug)
 	    krb_warning("Getting host entry for %s...", k_host->host);
@@ -117,22 +137,35 @@ send_to_kdc(KTEXT pkt, KTEXT rpkt, char *realm)
 	    krb_warning("%s.\n",
 			host ? "Got it" : "Didn't get it");
         }
-        if (!host)
+        if (host == NULL)
             continue;
         no_host = 0;    /* found at least one */
-	while ((p = *(host->h_addr_list)++)) {
-	    hosts = realloc(hosts, sizeof(*hosts) * (n_hosts + 1));
-	    if (hosts == NULL)
-		return SKDC_CANT;
-	    memset (&hosts[n_hosts].addr, 0, sizeof(hosts[n_hosts].addr));
-	    hosts[n_hosts].addr.sin_family = host->h_addrtype;
-	    hosts[n_hosts].addr.sin_port = htons(k_host->port);
-	    hosts[n_hosts].proto = k_host->proto;
-	    memcpy(&hosts[n_hosts].addr.sin_addr, p,
-		   sizeof(hosts[n_hosts].addr.sin_addr));
-	    ++n_hosts;
-	    if (send_recv(pkt, rpkt, hosts[n_hosts-1].proto,
-			  &hosts[n_hosts-1].addr, hosts, n_hosts)) {
+
+	n_addrs = 0;
+	for (addr_list = host->h_addr_list; *addr_list != NULL; ++addr_list)
+	    ++n_addrs;
+
+	tmp = realloc (hosts, (n_hosts + n_addrs) * sizeof(*hosts));
+	if (tmp == NULL) {
+	    free (hosts);
+	    return SKDC_CANT;
+	}
+	hosts = tmp;
+
+	for (addr_list = host->h_addr_list, j = 0;
+	     (p = *addr_list) != NULL;
+	     ++addr_list, ++j) {
+	    memset (&hosts[n_hosts + j].addr, 0, sizeof(struct sockaddr_in));
+	    hosts[n_hosts + j].addr.sin_family = host->h_addrtype;
+	    hosts[n_hosts + j].addr.sin_port   = htons(k_host->port);
+	    hosts[n_hosts + j].proto           = k_host->proto;
+	    memcpy(&hosts[n_hosts + j].addr.sin_addr, p,
+		   sizeof(struct in_addr));
+	}
+
+	for (j = 0; j < n_addrs; ++j) {
+	    if (send_recv(pkt, rpkt, hosts[n_hosts + j].proto,
+			  &hosts[n_hosts + j].addr)) {
 		retval = KSUCCESS;
 		goto rtn;
 	    }
@@ -140,11 +173,11 @@ send_to_kdc(KTEXT pkt, KTEXT rpkt, char *realm)
 		krb_warning("Timeout, error, or wrong descriptor\n");
 	    }
 	}
+	n_hosts += j;
     }
     if (no_host) {
 	if (krb_debug)
-	    krb_warning("%s: can't find any Kerberos host.\n",
-			prog);
+	    krb_warning("send_to_kdc: can't find any Kerberos host.\n");
         retval = SKDC_CANT;
         goto rtn;
     }
@@ -153,9 +186,7 @@ send_to_kdc(KTEXT pkt, KTEXT rpkt, char *realm)
 	for (i = 0; i < n_hosts; ++i) {
 	    if (send_recv(pkt, rpkt,
 			  hosts[i].proto,
-			  &hosts[i].addr,
-			  hosts,
-			  n_hosts)) {
+			  &hosts[i].addr)) {
 		retval = KSUCCESS;
 		goto rtn;
 	    }
@@ -167,12 +198,14 @@ rtn:
     return(retval);
 }
 
-static int udp_socket(void)
+static int
+udp_socket(void)
 {
     return socket(AF_INET, SOCK_DGRAM, 0);
 }
 
-static int udp_connect(int s, struct sockaddr_in *adr)
+static int
+udp_connect(int s, struct sockaddr_in *adr)
 {
     if(krb_debug) {
 	krb_warning("connecting to %s udp, port %d\n", 
@@ -182,7 +215,8 @@ static int udp_connect(int s, struct sockaddr_in *adr)
     return connect(s, (struct sockaddr*)adr, sizeof(*adr));
 }
 
-static int udp_send(int s, struct sockaddr_in* adr, KTEXT pkt)
+static int
+udp_send(int s, struct sockaddr_in* adr, KTEXT pkt)
 {
     if(krb_debug) {
 	krb_warning("sending %d bytes to %s, udp port %d\n", 
@@ -193,12 +227,14 @@ static int udp_send(int s, struct sockaddr_in* adr, KTEXT pkt)
     return send(s, pkt->dat, pkt->length, 0);
 }
 
-static int tcp_socket(void)
+static int
+tcp_socket(void)
 {
     return socket(AF_INET, SOCK_STREAM, 0);
 }
 
-static int tcp_connect(int s, struct sockaddr_in *adr)
+static int
+tcp_connect(int s, struct sockaddr_in *adr)
 {
     if(krb_debug) {
 	krb_warning("connecting to %s, tcp port %d\n", 
@@ -208,7 +244,8 @@ static int tcp_connect(int s, struct sockaddr_in *adr)
     return connect(s, (struct sockaddr*)adr, sizeof(*adr));
 }
 
-static int tcp_send(int s, struct sockaddr_in* adr, KTEXT pkt)
+static int
+tcp_send(int s, struct sockaddr_in* adr, KTEXT pkt)
 {
     unsigned char len[4];
     if(krb_debug) {
@@ -217,47 +254,64 @@ static int tcp_send(int s, struct sockaddr_in* adr, KTEXT pkt)
 		    inet_ntoa(adr->sin_addr), 
 		    ntohs(adr->sin_port));
     }
-    krb_put_int(pkt->length, len, 4);
+    krb_put_int(pkt->length, len, sizeof(len), 4);
     if(send(s, len, sizeof(len), 0) != sizeof(len))
 	return -1;
     return send(s, pkt->dat, pkt->length, 0);
 }
 
-static int udptcp_recv(void *buf, size_t len, KTEXT rpkt)
+static int
+udptcp_recv(void *buf, size_t len, KTEXT rpkt)
 {
+    int pktlen = min(len, MAX_KTXT_LEN);
+
     if(krb_debug)
-	krb_warning("recieved %d bytes on udp/tcp socket\n", len);
-    memcpy(rpkt->dat, buf, len);
-    rpkt->length = len;
+	krb_warning("recieved %lu bytes on udp/tcp socket\n", 
+		    (unsigned long)len);
+    memcpy(rpkt->dat, buf, pktlen);
+    rpkt->length = pktlen;
     return 0;
 }
 
-static int url_parse(const char *url, char *host, size_t len, short *port)
+static int
+url_parse(const char *url, char *host, size_t len, short *port)
 {
     const char *p;
+    size_t n;
+
     if(strncmp(url, "http://", 7))
 	return -1;
     url += 7;
-    strncpy(host, url, len);
     p = strchr(url, ':');
-    if(p){
-	*port = atoi(p+1);
-	if(p - url >= len)
+    if(p) {
+	char *end;
+
+	*port = htons(strtol(p + 1, &end, 0));
+	if (end == p + 1)
 	    return -1;
-	host[p - url] = 0;
-    }else{
-	*port = 80;
-	host[len - 1] = 0;
+	n = p - url;
+    } else {
+	*port = k_getportbyname ("http", "tcp", htons(80));
+	p = strchr(url, '/');
+	if (p)
+	    n = p - url;
+	else
+	    n = strlen(url);
     }
+    if (n >= len)
+	return -1;
+    memcpy(host, url, n);
+    host[n] = '\0';
     return 0;
 }
 
 #define PROXY_VAR "krb4_proxy"
 
-static int http_connect(int s, struct sockaddr_in *adr)
+static int
+http_connect(int s, struct sockaddr_in *adr)
 {
     const char *proxy = krb_get_config_string(PROXY_VAR);
-    char host[MaxHostNameLen + 1];
+    char host[MaxHostNameLen];
     short port;
     struct hostent *hp;
     struct sockaddr_in sin;
@@ -274,10 +328,10 @@ static int http_connect(int s, struct sockaddr_in *adr)
     memset(&sin, 0, sizeof(sin));
     sin.sin_family = AF_INET;
     memcpy(&sin.sin_addr, hp->h_addr, sizeof(sin.sin_addr));
-    sin.sin_port = htons(port);
+    sin.sin_port = port;
     if(krb_debug) {
 	krb_warning("connecting to proxy on %s (%s) port %d\n", 
-		    host, inet_ntoa(sin.sin_addr), port);
+		    host, inet_ntoa(sin.sin_addr), ntohs(port));
     }
     return connect(s, (struct sockaddr*)&sin, sizeof(sin));
 }
@@ -290,7 +344,7 @@ http_send(int s, struct sockaddr_in* adr, KTEXT pkt)
 
     if(base64_encode(pkt->dat, pkt->length, &str) < 0)
 	return -1;
-    if(krb_get_config_string(PROXY_VAR)){
+    if(krb_get_config_string(PROXY_VAR)) {
 	if(krb_debug) {
 	    krb_warning("sending %d bytes to %s, tcp port %d (via proxy)\n", 
 			pkt->length,
@@ -301,7 +355,7 @@ http_send(int s, struct sockaddr_in* adr, KTEXT pkt)
 		 inet_ntoa(adr->sin_addr),
 		 ntohs(adr->sin_port),
 		 str);
-    }else {
+    } else {
 	if(krb_debug) {
 	    krb_warning("sending %d bytes to %s, http port %d\n", 
 			pkt->length,
@@ -340,7 +394,17 @@ http_recv(void *buf, size_t len, KTEXT rpkt)
     }
     p += 4;
     if(krb_debug)
-	krb_warning("recieved %d bytes on http socket\n", (tmp + len) - p);
+	krb_warning("recieved %lu bytes on http socket\n", 
+		    (unsigned long)((tmp + len) - p));
+    if((tmp + len) - p > MAX_KTXT_LEN) {
+	free(tmp);
+	return -1;
+    }
+    if (strncasecmp (tmp, "HTTP/1.0 2", 10) != 0
+	&& strncasecmp (tmp, "HTTP/1.1 2", 10) != 0) {
+	free (tmp);
+	return -1;
+    }
     memcpy(rpkt->dat, p, (tmp + len) - p);
     rpkt->length = (tmp + len) - p;
     free(tmp);
@@ -361,8 +425,7 @@ static struct proto_descr {
 };
 
 static int
-send_recv(KTEXT pkt, KTEXT rpkt, int proto, struct sockaddr_in *adr,
-	  struct host *addrs, int n_hosts)
+send_recv(KTEXT pkt, KTEXT rpkt, int proto, struct sockaddr_in *adr)
 {
     int i;
     int s;
@@ -389,7 +452,7 @@ send_recv(KTEXT pkt, KTEXT rpkt, int proto, struct sockaddr_in *adr,
 	fd_set readfds;
 	struct timeval timeout;
 	int len;
-	timeout.tv_sec = CLIENT_KRB_TIMEOUT;
+	timeout.tv_sec = client_timeout;
 	timeout.tv_usec = 0;
 	FD_ZERO(&readfds);
 	FD_SET(s, &readfds);
@@ -404,10 +467,14 @@ send_recv(KTEXT pkt, KTEXT rpkt, int proto, struct sockaddr_in *adr,
 	    return FALSE;
 	}
 	len = recv(s, buf + offset, sizeof(buf) - offset, 0);
-	if(len <= 0)
+	if (len < 0) {
+	    close(s);
+	    return FALSE;
+	}
+	if(len == 0)
 	    break;
 	offset += len;
-    }while(protos[i].stream_flag);
+    } while(protos[i].stream_flag);
     close(s);
     if((*protos[i].recv)(buf, offset, rpkt) < 0)
 	return FALSE;
