@@ -9,7 +9,7 @@
 #include "config.h"
 #include "protos.h"
 
-RCSID("$Id: kerberos.c,v 1.76 1997/12/15 00:01:40 assar Exp $");
+RCSID("$Id: kerberos.c,v 1.84.2.1 1999/07/22 03:18:03 assar Exp $");
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -54,7 +54,7 @@ RCSID("$Id: kerberos.c,v 1.76 1997/12/15 00:01:40 assar Exp $");
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
-#if defined(HAVE_SYS_IOCTL_H) && SunOS != 4
+#if defined(HAVE_SYS_IOCTL_H) && SunOS != 40
 #include <sys/ioctl.h>
 #endif
 #ifdef HAVE_SYS_FILIO_H
@@ -88,7 +88,6 @@ static des_cblock master_key;
 
 static struct timeval kerb_time;
 static u_char master_key_version;
-static char k_instance[INST_SZ];
 static char *lt;
 static int more;
 
@@ -134,8 +133,8 @@ kerb_err_reply(int f, struct sockaddr_in *client, int err, char *string)
     KTEXT   e_pkt = &e_pkt_st;
     static char e_msg[128];
 
-    strcpy(e_msg, "\nKerberos error -- ");
-    strcat(e_msg, string);
+    snprintf (e_msg, sizeof(e_msg),
+	      "\nKerberos error -- %s", string);
     cr_err_reply(e_pkt, req_name_ptr, req_inst_ptr, req_realm_ptr,
 		 req_time_ws, err, e_msg);
     sendto(f, (char*)e_pkt->dat, e_pkt->length, 0, (struct sockaddr *)client,
@@ -246,7 +245,7 @@ set_tgtkey(char *r)
     copy_to_key(&p->key_low, &p->key_high, key);
     unseal(&key);
     krb_set_key(key, 0);
-    strcpy(lastrealm, r);
+    strcpy_truncate (lastrealm, r, REALM_SZ);
     return (KSUCCESS);
 }
 
@@ -277,7 +276,9 @@ kerberos(unsigned char *buf, int len,
     
     unsigned char *p = buf;
     if(len < 2){
-	strcpy((char*)rpkt->dat, "Packet too short");
+	strcpy_truncate((char*)rpkt->dat,
+			"Packet too short",
+			sizeof(rpkt->dat));
 	return KFAILURE;
     }
 
@@ -286,7 +287,9 @@ kerberos(unsigned char *buf, int len,
     pvno = *p++;
     if(pvno != KRB_PROT_VERSION){
 	msg = klog(L_KRB_PERR, "KRB protocol version mismatch (%d)", pvno);
-	strcpy((char*)rpkt->dat, msg);
+	strcpy_truncate((char*)rpkt->dat,
+			msg,
+			sizeof(rpkt->dat));
 	return KERB_ERR_PKT_VER;
     }
     msg_type = *p++;
@@ -305,12 +308,16 @@ kerberos(unsigned char *buf, int len,
 	     inet_ntoa(client->sin_addr),
 	     proto, ntohs(server->sin_port));
 	if((err = check_princ(name, inst, 0, &a_name))){
-	    strcpy((char*)rpkt->dat, krb_get_err_text(err));
+	    strcpy_truncate((char*)rpkt->dat,
+			    krb_get_err_text(err),
+			    sizeof(rpkt->dat));
 	    return err;
 	}
 	tk->length = 0;
 	if((err = check_princ(service, sinst, 0, &s_name))){
-	    strcpy((char*)rpkt->dat, krb_get_err_text(err));
+	    strcpy_truncate((char*)rpkt->dat,
+			    krb_get_err_text(err),
+			    sizeof(rpkt->dat));
 	    return err;
 	}
 	life = min(life, s_name.max_life);
@@ -339,13 +346,15 @@ kerberos(unsigned char *buf, int len,
 	}
 	return 0;
     case AUTH_MSG_APPL_REQUEST:
-	strcpy(realm, (char*)buf + 3);
+	strcpy_truncate(realm, (char*)buf + 3, REALM_SZ);
 	if((err = set_tgtkey(realm))){
 	    msg = klog(L_ERR_UNK,
 		       "Unknown realm %s from %s (%s/%u)", 
 		       realm, inet_ntoa(client->sin_addr),
 		       proto, ntohs(server->sin_port));
-	    strcpy((char*)rpkt->dat, msg);
+	    strcpy_truncate((char*)rpkt->dat,
+			    msg,
+			    sizeof(rpkt->dat));
 	    return err;
 	}
 	p = buf + strlen(realm) + 4;
@@ -361,7 +370,9 @@ kerberos(unsigned char *buf, int len,
 		       proto,
 		       ntohs(server->sin_port),
 		       krb_get_err_text(err));
-	    strcpy((char*)rpkt->dat, msg);
+	    strcpy_truncate((char*)rpkt->dat,
+			    msg,
+			    sizeof(rpkt->dat));
 	    return err;
 	}
 	p += krb_get_int(p, &req_time, 4, lsb);
@@ -378,19 +389,24 @@ kerberos(unsigned char *buf, int len,
 	if(strcmp(ad.prealm, realm)){
 	    msg = klog(L_ERR_UNK, "Can't hop realms: %s -> %s", 
 		       realm, ad.prealm);
-	    strcpy((char*)rpkt->dat, msg);
+	    strcpy_truncate((char*)rpkt->dat,
+			    msg,
+			    sizeof(rpkt->dat));
 	    return KERB_ERR_PRINCIPAL_UNKNOWN;
 	}
 
 	if(!strcmp(service, "changepw")){
-	    strcpy((char*)rpkt->dat, 
-		   "Can't authorize password changed based on TGT");
+	    strcpy_truncate((char*)rpkt->dat, 
+			    "Can't authorize password changed based on TGT",
+			    sizeof(rpkt->dat));
 	    return KERB_ERR_PRINCIPAL_UNKNOWN;
 	}
 
 	err = check_princ(service, sinst, life, &s_name);
 	if(err){
-	    strcpy((char*)rpkt->dat, krb_get_err_text(err));
+	    strcpy_truncate((char*)rpkt->dat,
+			    krb_get_err_text(err),
+			    sizeof(rpkt->dat));
 	    return err;
 	}
 	life = min(life, 
@@ -433,7 +449,9 @@ kerberos(unsigned char *buf, int len,
 		   inet_ntoa(client->sin_addr),
 		   proto,
 		   ntohs(server->sin_port));
-	strcpy((char*)rpkt->dat, msg);
+	strcpy_truncate((char*)rpkt->dat,
+			msg,
+			sizeof(rpkt->dat));
 	return KFAILURE;
     }
 }
@@ -451,7 +469,7 @@ kerberos_wrap(int s, KTEXT data, char *proto, struct sockaddr_in *client,
     if(http_flag){
 	const char *msg = 
 	    "HTTP/1.1 200 OK\r\n"
-	    "Server: KTH-KRB/" VERSION "\r\n"
+	    "Server: KTH-KRB/1\r\n"
 	    "Content-type: application/octet-stream\r\n"
 	    "Content-transfer-encoding: binary\r\n\r\n";
 	sendto(s, msg, strlen(msg), 0, (struct sockaddr *)client,
@@ -580,7 +598,7 @@ add_port(struct port_spec **ports, int *num_ports, int port, int type)
 }
 
 static void
-make_sockets(char *port_spec, struct in_addr *i_addr, 
+make_sockets(const char *port_spec, struct in_addr *i_addr, 
 	     struct descr **fds, int *nfds)
 {
     int tp;
@@ -590,9 +608,12 @@ make_sockets(char *port_spec, struct in_addr *i_addr,
     struct port_spec *ports = NULL;
     int num_ports = 0;
     int i, j;
-    
+    char *port_spec_copy = strdup (port_spec);
 
-    for(p = strtok_r(port_spec, ", \t", &pos); 
+    if (port_spec_copy == NULL)
+	err (1, "strdup");
+
+    for(p = strtok_r(port_spec_copy, ", \t", &pos); 
 	p; 
 	p = strtok_r(NULL, ", \t", &pos)){
 	if(strcmp(p, "+") == 0){
@@ -626,6 +647,7 @@ make_sockets(char *port_spec, struct in_addr *i_addr,
 	    }
 	}
     }
+    free (port_spec_copy);
 
     if(num_ports == 0)
 	errx(1, "No valid ports specified!");
@@ -634,8 +656,8 @@ make_sockets(char *port_spec, struct in_addr *i_addr,
 	*nfds = 1;
 	a = malloc(sizeof(*a) * *nfds);
 	if (a == NULL)
-	    errx (1, "Failed to allocate %u bytes",
-		  sizeof(*a) * *nfds);
+	    errx (1, "Failed to allocate %lu bytes",
+		  (unsigned long)(sizeof(*a) * *nfds));
 	memcpy(a, i_addr, sizeof(struct in_addr));
     } else
 	*nfds = k_get_all_addrs (&a);
@@ -648,14 +670,14 @@ make_sockets(char *port_spec, struct in_addr *i_addr,
 	*nfds = 1;
 	a = malloc(sizeof(*a) * *nfds);
 	if (a == NULL)
-	    errx (1, "Failed to allocate %u bytes",
-		  sizeof(*a) * *nfds);
+	    errx (1, "Failed to allocate %lu bytes",
+		  (unsigned long)(sizeof(*a) * *nfds));
 	memcpy(a, &any, sizeof(struct in_addr));
     }
     *fds = malloc(*nfds * num_ports * sizeof(**fds));
     if (*fds == NULL)
-	errx (1, "Failed to allocate %u bytes",
-	      *nfds * num_ports * sizeof(**fds));
+	errx (1, "Failed to allocate %lu bytes",
+	      (unsigned long)(*nfds * num_ports * sizeof(**fds)));
     for (i = 0; i < *nfds; i++) {
 	for(j = 0; j < num_ports; j++) {
 	    mksocket(*fds + num_ports * i + j, a[i], 
@@ -692,7 +714,7 @@ main(int argc, char **argv)
 	     * Set parameters to slave server defaults.
 	     */
 	    if (max_age == -1 && !nflag)
-		max_age = ONE_DAY;	/* 24 hours */
+		max_age = THREE_DAYS;	/* Survive weekend */
 	    if (pause_int == -1)
 		pause_int = FIVE_MINUTES; /* 5 minutes */
 	    break;
@@ -703,29 +725,41 @@ main(int argc, char **argv)
 	case 'm':
 	    mflag++;		/* running manually; prompt for master key */
 	    break;
-	case 'p':
+	case 'p': {
 	    /* Set pause interval. */
-	    if (!isdigit(optarg[0]))
-		usage();
-	    pause_int = atoi(optarg);
+	    char *tmp;
+
+	    pause_int = strtol (optarg, &tmp, 0);
+	    if (pause_int == 0 && tmp == optarg) {
+		fprintf(stderr, "pause_int `%s' not a number\n", optarg);
+		usage ();
+	    }
+
 	    if ((pause_int < 5) ||  (pause_int > ONE_HOUR)) {
 		fprintf(stderr, "pause_int must be between 5 and 3600 seconds.\n");
 		usage();
 	    }
 	    break;
+	}
 	case 'P':
 	    port_spec = optarg;
 	    break;
-	case 'a':
+	case 'a': {
 	    /* Set max age. */
-	    if (!isdigit(optarg[0])) 
-		usage();
-	    max_age = atoi(optarg);
+	    char *tmp;
+
+	    max_age = strtol (optarg, &tmp, 0);
+	    if (max_age == 0 && tmp == optarg) {
+		fprintf (stderr, "max_age `%s' not a number\n", optarg);
+		usage ();
+	    }
 	    if ((max_age < ONE_HOUR) || (max_age > THREE_DAYS)) {
-		fprintf(stderr, "max_age must be between one hour and three days, in seconds\n");
+		fprintf(stderr, "max_age must be between one hour and "
+			"three days, in seconds\n");
 		usage();
 	    }
 	    break;
+	}
 	case 'l':
 	    /* Set alternate log file */
 	    log_file = optarg;
@@ -733,7 +767,7 @@ main(int argc, char **argv)
 	case 'r':
 	    /* Set realm name */
 	    rflag++;
-	    strcpy(local_realm, optarg);
+	    strcpy_truncate(local_realm, optarg, sizeof(local_realm));
 	    break;
 	case 'i':
 	    /* Only listen on this address */
@@ -775,10 +809,6 @@ main(int argc, char **argv)
 
     kset_logfile(log_file);
     
-    /* find our hostname, and use it as the instance */
-    if (gethostname(k_instance, INST_SZ))
-	err (1, "gethostname");
-
     make_sockets(port_spec, i_flag ? &i_addr : NULL, &fds, &nfds);
 
     /* do all the database and cache inits */
@@ -874,7 +904,7 @@ read_socket(struct descr *n)
 	    char *save = NULL;
 
 	    n->buf.dat[n->buf.length - 1] = 0;
-	    strtok_r(n->buf.dat, " \t\r\n", &save);
+	    strtok_r((char *)n->buf.dat, " \t\r\n", &save);
 	    p = strtok_r(NULL, " \t\r\n", &save);
 	    if(p == NULL)
 		p = "";
@@ -883,7 +913,7 @@ read_socket(struct descr *n)
 	    if(n->buf.length <= 0){
 		const char *msg = 
 		    "HTTP/1.1 404 Not found\r\n"
-		    "Server: KTH-KRB/" VERSION "\r\n"
+		    "Server: KTH-KRB/1\r\n"
 		    "Content-type: text/html\r\n"
 		    "Content-transfer-encoding: 8bit\r\n\r\n"
 		    "<TITLE>404 Not found</TITLE>\r\n"
